<?php

declare(strict_types = 1);

namespace Drupal\data_pipelines\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityConstraintViolationListInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\DatasetPipelineInterface;
use Drupal\data_pipelines\DatasetPipelinePluginManager;
use Drupal\data_pipelines\Source\DatasetSourceInterface;
use Drupal\data_pipelines\Source\DatasetSourcePluginManager;

/**
 * Defines a content entity for datasets.
 *
 * The bundles of this content entity are defined by the data_pipelines_source
 * plugin manager.
 *
 * @ContentEntityType(
 *   id = "data_pipelines",
 *   label = @Translation("Dataset"),
 *   label_collection = @Translation("Datasets"),
 *   label_singular = @Translation("dataset"),
 *   label_plural = @Translation("datasets"),
 *   label_count = @PluralTranslation(
 *     singular = "@count dataset",
 *     plural = "@count datasets",
 *   ),
 *   bundle_label = @Translation("Dataset source"),
 *   admin_permission = "data_pipelines list",
 *   base_table = "data_pipelines",
 *   translatable = FALSE,
 *   bundle_plugin_type = "data_pipelines_source",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "source",
 *     "uuid" = "uuid",
 *     "label" = "name",
 *   },
 *   handlers = {
 *     "storage" = \Drupal\data_pipelines\EntityHandlers\DatasetStorage::class,
 *     "list_builder" = \Drupal\data_pipelines\EntityHandlers\DatasetListBuilder::class,
 *     "views_data" = \Drupal\views\EntityViewsData::class,
 *     "route_provider" = {
 *       "html" = \Drupal\data_pipelines\EntityHandlers\DatasetRouteProvider::class,
 *     },
 *     "view_builder" = \Drupal\Core\Entity\EntityViewBuilder::class,
 *     "access" = \Drupal\data_pipelines\EntityHandlers\DatasetAccessHandler::class,
 *     "form" = {
 *       "add" = \Drupal\data_pipelines\Form\DatasetForm::class,
 *       "edit" = \Drupal\data_pipelines\Form\DatasetForm::class,
 *       "delete" = \Drupal\Core\Entity\ContentEntityDeleteForm::class,
 *       "index" = \Drupal\data_pipelines\Form\DatasetIndexForm::class,
 *     },
 *   },
 *   links = {
 *     "collection" = "/admin/content/datasets",
 *     "edit-form" = "/admin/content/datasets/manage/{data_pipelines}/edit",
 *     "canonical" = "/admin/content/datasets/manage/{data_pipelines}",
 *     "add-page" = "/admin/content/datasets/add",
 *     "add-form" = "/admin/content/datasets/add/{source}",
 *     "delete-form" = "/data_pipelines/{data_pipelines}/delete",
 *     "index-form" = "/data_pipelines/{data_pipelines}/index",
 *   }
 * )
 */
class Dataset extends ContentEntityBase implements DatasetInterface {

  /**
   * {@inheritDoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['pipeline'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Pipeline'))
      ->setDescription(t('The processing pipeline for the dataset.'))
      ->setRequired(TRUE)
      ->setSetting('allowed_values_function', self::class . '::getPipelines')
      ->setDisplayOptions('form', [
        'type' => 'options_select',
      ]);
    $fields['status'] = BaseFieldDefinition::create('list_string')
      ->setLabel(new TranslatableMarkup('Status'))
      ->setDefaultValue(self::STATUS_PENDING_VALIDATION)
      ->setSetting('allowed_values', [
        self::STATUS_PENDING_VALIDATION => new TranslatableMarkup('Pending validation'),
        self::STATUS_PENDING_INDEX => new TranslatableMarkup('Pending indexing'),
        self::STATUS_INDEXED => new TranslatableMarkup('Indexed'),
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'list_default',
        'label' => 'above',
        'weight' => -4,
      ])
      ->setRequired(TRUE);
    $fields['log'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Log'))
      ->setDescription(t('Log entries.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'basic_string',
        'label' => 'above',
        'weight' => 40,
      ])
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE);
    return $fields;
  }

  /**
   * An method to get the plugins.
   *
   * The plugins are keyed using the plugin Id and the value being the plugin
   * label.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param \Drupal\data_pipelines\Entity\Dataset $dataset
   *   The dataset entity.
   *
   * @return array
   *   The plugins.
   */
  public static function getPipelines(FieldDefinitionInterface $field_definition, Dataset $dataset): array {
    return array_map(function (array $definition) {
      return $definition['label'];
    }, \Drupal::service('plugin.manager.data_pipelines_pipeline')->getDefinitions());
  }

  /**
   * Gets the source ID.
   *
   * @return string
   *   The source.
   */
  protected function getSourceId(): string {
    return $this->get('source')->value;
  }

  /**
   * Gets the pipeline ID.
   *
   * @return string
   *   The plugin.
   */
  protected function getPipelineId(): string {
    return $this->get('pipeline')->value;
  }

  /**
   * Gets the pipeline plugin.
   *
   * @return \Drupal\data_pipelines\DatasetPipelineInterface
   *   Pipeline.
   */
  protected function getPipeline(): DatasetPipelineInterface {
    return \Drupal::service('plugin.manager.data_pipelines_pipeline')->createInstance($this->getPipelineId());
  }

  /**
   * Gets the source plugin.
   *
   * @return \Drupal\data_pipelines\Source\DatasetSourceInterface
   *   Source plugin.
   */
  protected function getSource(): DatasetSourceInterface {
    return \Drupal::service('plugin.manager.data_pipelines_source')->createInstance($this->getSourceId());
  }

  /**
   * {@inheritdoc}
   */
  public function validate(): EntityConstraintViolationListInterface {
    $violations = parent::validate();
    if ($this->isSyncing()) {
      return $violations;
    }
    $pipeline = $this->getPipeline();
    $source = $this->getSource();
    foreach ($source->extractDataFromDataSet($this) as $data) {
      $violations->addAll($pipeline->validate($data));
    }
    return $violations;
  }

  /**
   * {@inheritdoc}
   */
  public function data(): \Generator {
    $source = $this->getSource();
    $pipeline = $this->getPipeline();
    foreach ($source->extractDataFromDataSet($this) as $data) {
      yield $pipeline->transform($data);
    }
  }

  /**
   * Performs a save operation inside a sync.
   *
   * @return $this
   */
  protected function saveWithSyncing(): DatasetInterface {
    $this->setSyncing(TRUE);
    $this->save();
    $this->setSyncing(FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function resetLogs(bool $auto_save = TRUE): DatasetInterface {
    $this->log = NULL;
    if ($auto_save) {
      $this->saveWithSyncing();
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function addLogMessage(string $message, bool $auto_save = TRUE): DatasetInterface {
    $this->log[] = $message;
    if ($auto_save) {
      $this->saveWithSyncing();
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getStatusLabel(): TranslatableMarkup {
    $map = $this->getFieldDefinition('status')->getSetting('allowed_values');
    return $map[$this->get('status')->value] ?? new TranslatableMarkup('Unknown');
  }

  /**
   * {@inheritdoc}
   */
  public function getLogs(): array {
    return array_column($this->get('log')->getValue(), 'value');
  }

  /**
   * {@inheritdoc}
   */
  public function setPendingValidation(): DatasetInterface {
    $this->status = self::STATUS_PENDING_VALIDATION;
    $this->saveWithSyncing();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setPendingIndex(): DatasetInterface {
    $this->status = self::STATUS_PENDING_INDEX;
    $this->saveWithSyncing();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setIndexingComplete(): DatasetInterface {
    $this->status->value = self::STATUS_INDEXED;
    $this->saveWithSyncing();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPendingValidation(): bool {
    return $this->status->value === self::STATUS_PENDING_VALIDATION;
  }

  /**
   * {@inheritdoc}
   */
  public function isPendingIndex(): bool {
    return $this->status->value === self::STATUS_PENDING_INDEX;
  }

  /**
   * {@inheritdoc}
   */
  public function isIndexed(): bool {
    return $this->status->value === self::STATUS_INDEXED;
  }

  /**
   * {@inheritdoc}
   */
  public function getPipelineLabel() {
    $manager = \Drupal::service('plugin.manager.data_pipelines_pipeline');
    assert($manager instanceof DatasetPipelinePluginManager);
    $plugin_id = $this->getPipelineId();
    if ($manager->hasDefinition($plugin_id)) {
      return $manager->getDefinition($plugin_id)['label'];
    }
    return $plugin_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceLabel() {
    $manager = \Drupal::service('plugin.manager.data_pipelines_source');
    assert($manager instanceof DatasetSourcePluginManager);
    $plugin_id = $this->getSourceId();
    return $manager->getDefinition($plugin_id)['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
    \Drupal::service('plugin.manager.queue_worker')->clearCachedDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);
    \Drupal::service('plugin.manager.queue_worker')->clearCachedDefinitions();
  }

}
