<?php

declare(strict_types = 1);

namespace Drupal\data_pipelines\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\data_pipelines\Entity\DatasetInterface;

/**
 * Provides a form for adding/editing datasets.
 *
 * @codeCoverageIgnore
 * @see \Drupal\Tests\data_pipelines\Functional\DatasetUiTest
 */
class DatasetForm extends ContentEntityForm {

  /**
   * Flag to track if validation is occuring.
   *
   * @var bool
   */
  protected $isValidating = FALSE;

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    $entity = parent::buildEntity($form, $form_state);
    if ($this->isValidating) {
      // We set the syncing flag to TRUE to avoid running validation of the data
      // from the form operation. As if the dataset is large, we may be unable
      // to complete validation in a single request.
      $entity->setSyncing(TRUE);
    }
    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->isValidating = TRUE;
    $entity = parent::validateForm($form, $form_state);
    $this->isValidating = FALSE;
    $entity->setSyncing(FALSE);
    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $dataset = $this->entity;
    assert($dataset instanceof DatasetInterface);

    $result = $dataset->save();

    $edit_link = $dataset->toLink($this->t('Edit'), 'edit-form')->toString();
    $view_link = $dataset->toLink()->toString();
    switch ($result) {
      case SAVED_NEW:
        $this->messenger()->addStatus($this->t('Created dataset %dataset.', ['%dataset' => $view_link]));
        $this->logger('data_pipelines')->notice('Created dataset %dataset.', ['%dataset' => $dataset->label(), 'link' => $edit_link]);
        break;

      case SAVED_UPDATED:
        $this->messenger()->addStatus($this->t('Saved dataset %dataset.', ['%dataset' => $view_link]));
        $this->logger('data_pipelines')->notice('Saved dataset %dataset.', ['%dataset' => $dataset->label(), 'link' => $edit_link]);
        break;
    }
    batch_set(DatasetBatchOperations::batchForDataset($dataset));
    $form_state->setRedirectUrl($dataset->toUrl('collection'));
    return $result;
  }

}
