<?php

declare(strict_types=1);

namespace Drupal\data_pipelines\Plugin\DatasetSource;

use Drupal\data_pipelines\Source\DatasetSourceBase;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\link\LinkItemInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\BadResponseException;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a base plugin for data sources that fetch from a URI.
 */
abstract class UriBase extends DatasetSourceBase implements ContainerFactoryPluginInterface {

  /**
   * Cache lifetime.
   */
  const CACHE_LIFETIME = 86400;

  /**
   * Guzzle client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * Cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new UriBase.
   *
   * @param array $configuration
   *   Configuration.
   * @param string $plugin_id
   *   Plugin ID.
   * @param array $plugin_definition
   *   Definition.
   * @param \GuzzleHttp\ClientInterface $client
   *   HTTP Client.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   Cache backend.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   Time service.
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, ClientInterface $client, CacheBackendInterface $cacheBackend, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->cacheBackend = $cacheBackend;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('cache.default'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildFieldDefinitions(): array {
    return [
      $this->getFieldName() => BaseFieldDefinition::create('link')
        ->setLabel(new TranslatableMarkup('Remote URL'))
        ->setDescription(new TranslatableMarkup('The URL of the remote dataset.'))
        ->setRequired(TRUE)
        ->setSettings([
          'link_type' => LinkItemInterface::LINK_EXTERNAL,
          'title' => DRUPAL_DISABLED,
        ])
        ->setDisplayOptions('form', [
          'type' => 'link_default',
        ]
      ),
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getResource(DatasetInterface $dataset) {
    // Check for cache first.
    $cid = sprintf('dataset:%s', $dataset->uuid());
    if ($cached = $this->cacheBackend->get($cid)) {
      return $this->resourceFromString($cached->data);
    }
    try {
      $uri = $dataset->get($this->getFieldName())->uri;
      $response = $this->client->request('GET', $uri);
      $data = (string) $response->getBody();
      $this->cacheBackend->set($cid, (string) $data, $this->time->getCurrentTime() + self::CACHE_LIFETIME, $dataset->getCacheTags());
      return $this->resourceFromString($data);
    }
    catch (BadResponseException $e) {
      $response = $e->getResponse();
      $dataset->addLogMessage(sprintf('Error fetching %s: %s - %s', $uri, $response->getStatusCode(), $response->getReasonPhrase()));
      return NULL;
    }
    catch (GuzzleException $e) {
      $dataset->addLogMessage(sprintf('Error fetching %s: %s', $uri, $e->getMessage()));
      return NULL;
    }
  }

  /**
   * Gets a resource from a string.
   *
   * @param string $data
   *   Data to convert into a resource.
   *
   * @return resource|null
   *   Resource, or NULL on failure.
   */
  protected function resourceFromString(string $data) {
    if ($resource = fopen('php://temp', 'r+')) {
      fwrite($resource, $data);
      rewind($resource);
      return $resource;
    }
    return NULL;
  }

}
