<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel;

use Drupal\data_pipelines\Entity\Dataset;

/**
 * Defines a test for DatasetPipelineBase.
 *
 * @group data_pipelines
 * @covers \Drupal\data_pipelines\DatasetPipelineBase
 * @covers \Drupal\data_pipelines\Entity\Dataset::validate
 * @covers \Drupal\data_pipelines\Plugin\DataType\DatasetData
 * @covers \Drupal\data_pipelines\Plugin\Validation\Constraint\ItemCountValidator
 */
class DatasetPipelineBaseTest extends DatasetKernelTestBase {

  /**
   * Tests validation on a valid CSV.
   */
  public function testValidationValidCsv() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'test_pipeline_1',
      'csv_file' => $file,
    ]);
    $this->assertEmpty($dataset->validate());
  }

  /**
   * Tests validation on a valid JSON file.
   */
  public function testValidationValidJson() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1.json');
    $dataset = Dataset::create([
      'source' => 'json_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'test_pipeline_1',
      'json_file' => $file,
    ]);
    $this->assertEmpty($dataset->validate());
  }

  /**
   * Tests validation on an invalid CSV.
   */
  public function testValidationInvalidCsv() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1-invalid.csv');
    $dataset = Dataset::create([
      'source' => 'csv_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'test_pipeline_1',
      'csv_file' => $file,
    ]);
    $result = $dataset->validate();
    $this->assertCount(2, $result);
    $violation = $result->get(0);
    $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('jo', $violation->getInvalidValue());
    $violation = $result->get(1);
    $this->assertEquals('Firstname is required', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('', $violation->getInvalidValue());
    // If the entity is syncing, we don't validate the dataset.
    $dataset->setSyncing(TRUE);
    $result = $dataset->validate();
    $this->assertCount(0, $result);
  }

  /**
   * Tests validation on an invalid JSON file.
   */
  public function testValidationInvalidJson() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1-invalid.json');
    $dataset = Dataset::create([
      'source' => 'json_file',
      'pipeline' => 'test_pipeline_1',
      'name' => $this->randomMachineName(),
      'json_file' => $file,
    ]);
    $result = $dataset->validate();
    $this->assertCount(3, $result);
    $violation = $result->get(0);
    $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('jo', $violation->getInvalidValue());
    $violation = $result->get(1);
    $this->assertEquals('Firstname is required', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('', $violation->getInvalidValue());
    $violation = $result->get(2);
    $this->assertEquals('All records should have 3 values', $violation->getMessage());
    $this->assertEquals('', $violation->getPropertyPath());
    $this->assertEquals([
      'firstname' => 'missing',
      'lastname' => 'a value',
    ], $violation->getInvalidValue());
  }

}
