<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel\Form;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Form\DatasetBatchOperations;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;

/**
 * Defines a class for testing DatasetBatchOperations.
 *
 * @covers \Drupal\data_pipelines\Form\DatasetBatchOperations
 */
class DatasetBatchOperationsTest extends DatasetKernelTestBase {

  /**
   * Tests batch builder.
   */
  public function testBatchForDataset(): void {
    $dataset = $this->createTestDataset();
    $batch = DatasetBatchOperations::batchForDataset($dataset);
    $this->assertEquals([
      [[DatasetBatchOperations::class, 'operationValidate'], [$dataset->id()]],
      [[DatasetBatchOperations::class, 'operationIndex'], [$dataset->id()]],
    ], $batch['operations']);
  }

  /**
   * Tests validation operation.
   */
  public function testValidation(): void {
    $dataset = $this->createTestDataset();
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationValidate((int) $dataset->id(), $context);
    $this->assertEquals('Validation complete', (string) $context['message']);
    $csv = <<<CSV
should_we,firstname,lastname
Y,jo,bloggs
N,betty,bloggs
CSV;
    $dataset = $this->createTestDataset([
      'csv_string' => $csv,
    ]);
    DatasetBatchOperations::operationValidate((int) $dataset->id(), $context);
    $dataset = $this->reloadDataset($dataset);
    $this->assertEquals('Validation complete, validation errors exist', (string) $context['message']);
    $this->assertEquals([
      'Firstname should be at least 3 characters',
    ], $dataset->getLogs());
    $this->assertTrue($dataset->isPendingValidation());
  }

  /**
   * Tests index operation with failed validation.
   */
  public function testIndexingWhenInvalid(): void {
    $dataset = $this->createTestDataset();
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationIndex((int) $dataset->id(), $context);
    $this->assertEquals('Skipped indexing dataset, validation errors exist.', (string) $context['message']);
  }

  /**
   * Tests index operation.
   */
  public function testIndexingValid(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_INDEX,
    ]);
    $queue = \Drupal::queue('data_pipelines_index:' . $dataset->id());
    $this->assertEquals(1, $queue->numberOfItems());
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::operationIndex((int) $dataset->id(), $context);
    $this->assertEquals('Indexed dataset.', (string) $context['message']);
    $this->assertEquals(0, $queue->numberOfItems());
  }

  /**
   * Tests finished with success.
   */
  public function testFinishedSuccess(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_INDEX,
    ]);
    $context = [
      'results' => [
        'dataset_id' => $dataset->id(),
      ],
    ];
    DatasetBatchOperations::operationIndex((int) $dataset->id(), $context);
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('Dataset %name was successfully indexed.', [
      '%name' => $dataset->label(),
    ]),
    ], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_STATUS));
  }

  /**
   * Tests finished without indexing.
   */
  public function testFinishedWithoutIndexing(): void {
    $dataset = $this->createTestDataset([
      'status' => DatasetInterface::STATUS_PENDING_INDEX,
    ]);
    $context = [
      'results' => [
        'dataset_id' => $dataset->id(),
      ],
    ];
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('Dataset %name was unable to be indexed. Please check <a href=":url">the logs</a>.', [
      ':url' => $dataset->toUrl()->toString(),
    ]),
    ], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_ERROR));
  }

  /**
   * Tests finished with error.
   */
  public function testFinishedWithError(): void {
    $context = [
      'results' => [],
    ];
    DatasetBatchOperations::finished(TRUE, $context['results']);
    $this->assertEquals([new TranslatableMarkup('The validation and indexing operation did not complete.')], \Drupal::messenger()->messagesByType(MessengerInterface::TYPE_ERROR));
  }

}
