<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel\Index;

use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\data_pipelines\DatasetData;
use Drupal\data_pipelines\Index\DatasetElasticsearchIndexer;
use Drupal\data_pipelines\Index\DatasetNotRemovedFromIndexException;
use Drupal\data_pipelines\Index\IndexingRequest;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;
use Elasticsearch\Client as BaseClient;
use Elasticsearch\Namespaces\IndicesNamespace;
use nodespark\DESConnector\Client;
use Prophecy\Argument;

/**
 * Defines a class for testing elastic indexer.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines\Index\DatasetElasticsearchIndexer
 * @covers \Drupal\data_pipelines\Index\DatasetNotRemovedFromIndexException
 */
class DatasetElasticsearchIndexerTest extends DatasetKernelTestBase {

  /**
   * Tests index creation.
   */
  public function testIndexCreation() {
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(FALSE);
    $indices->create([
      'index' => DatasetElasticsearchIndexer::INDEX_ID,
      'body' => [
        'settings' => [
          'number_of_shards' => DatasetElasticsearchIndexer::INDEX_SHARDS,
          'number_of_replicas' => DatasetElasticsearchIndexer::INDEX_REPLICAS,
          'codec' => DatasetElasticsearchIndexer::INDEX_CODEC,
        ],
      ],
    ])->shouldBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());
    $client->CheckResponseAck(Argument::any())->willReturn(TRUE);

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $dataset = $this->createTestDataset();
    $indexer->indexDataSet($dataset);
  }

  /**
   * Tests index not created if exists.
   */
  public function testIndexNotCreatedIfExists() {
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(TRUE);
    $indices->create(Argument::any())->shouldNotBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());
    $client->CheckResponseAck(Argument::any())->willReturn(TRUE);

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $dataset = $this->createTestDataset();
    $indexer->indexDataSet($dataset);
  }

  /**
   * Tests index creation failure.
   */
  public function testIndexCreationFailure() {
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willThrow(new \Exception('Barf'));
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $dataset = $this->createTestDataset();
    $this->assertFalse($indexer->indexDataSet($dataset));
  }

  /**
   * Tests index creation failure.
   */
  public function testIndexCreationAckFailure() {
    $indices = $this->prophesize(IndicesNamespace::class);
    $indices->exists(Argument::any())->willReturn(FALSE);
    $indices->create(Argument::any())->shouldBeCalled();
    $client = $this->prophesize(Client::class);
    $client->indices()->willReturn($indices->reveal());
    $client->CheckResponseAck(Argument::any())->willReturn(FALSE);

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $dataset = $this->createTestDataset();
    $this->assertFalse($indexer->indexDataSet($dataset));
  }

  /**
   * Tests indexing.
   */
  public function testIndexing() {
    $indices = $this->prophesize(IndicesNamespace::class);
    $dataset = $this->createTestDataset();
    $indices->exists(Argument::any())->willReturn(TRUE);
    $client = $this->prophesize(BaseClient::class);
    $client->indices()->willReturn($indices->reveal());
    $expected = [
      'index' => DatasetElasticsearchIndexer::INDEX_ID,
      'id' => $dataset->uuid(),
      'body' => [
        'data' => array_map(function (DatasetData $data) {
          return $data->getArrayCopy();
        }, iterator_to_array($dataset->data())),
      ],
    ];
    $client->index($expected)->shouldBeCalled();

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $indexer->indexDataSet($dataset);
  }

  /**
   * Tests deletion.
   */
  public function testDeleting() {
    $dataset = $this->createTestDataset();
    $client = $this->prophesize(BaseClient::class);
    $expected = [
      'index' => DatasetElasticsearchIndexer::INDEX_ID,
      'id' => $dataset->uuid(),
    ];
    $client->delete($expected)->shouldBeCalled();

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $indexer->deleteDataSetFromIndex(IndexingRequest::deleteDataSet($dataset));
  }

  /**
   * Tests deletion failure.
   */
  public function testDeletingException() {
    $dataset = $this->createTestDataset();
    $client = $this->prophesize(BaseClient::class);
    $expected = [
      'index' => DatasetElasticsearchIndexer::INDEX_ID,
      'id' => $dataset->uuid(),
    ];
    $client->delete($expected)->willThrow(new \Exception('barf'));

    $indexer = new DatasetElasticsearchIndexer($client->reveal(), $this->prophesize(LoggerChannelInterface::class)->reveal());
    $this->expectException(DatasetNotRemovedFromIndexException::class);
    $this->expectExceptionMessage(sprintf('Could not remove dataset %s from the index, error was barf', $dataset->label()));
    $indexer->deleteDataSetFromIndex(IndexingRequest::deleteDataSet($dataset));
  }

}
