<?php

declare(strict_types=1);

namespace src\Kernel;

use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\data_pipelines\Index\IndexingRequest;
use Drupal\data_pipelines\Plugin\QueueWorker\Indexer;
use Drupal\data_pipelines_test\MemoryDatasetIndexer;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;

/**
 * Defines a class for testing queue indexing.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines\EntityHandlers\DatasetStorage
 * @covers \Drupal\data_pipelines\Plugin\QueueWorker\Indexer
 */
class IndexingQueueTest extends DatasetKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->container->getDefinition('data_pipelines.indexer')->setClass(MemoryDatasetIndexer::class)->setFactory(NULL);
  }

  /**
   * Tests queue derivation.
   *
   * @covers \Drupal\data_pipelines\Plugin\Derivative\DatasetQueueWorkerDeriver
   */
  public function testQueueDerivation() {
    $queue = \Drupal::service('plugin.manager.queue_worker');
    assert($queue instanceof QueueWorkerManagerInterface);
    $this->assertCount(0, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_index') === 0;
    }));
    $dataset = $this->createTestDataset();
    $this->assertCount(1, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_index') === 0;
    }));
    $dataset->delete();
    $this->assertCount(0, array_filter(array_keys($queue->getDefinitions()), function (string $id) {
      return strpos($id, 'data_pipelines_index') === 0;
    }));
  }

  /**
   * Tests save and deletion.
   */
  public function testSaveAndDeletion() {
    $dataset = $this->createTestDataset();
    $queue = \Drupal::queue('data_pipelines_index:' . $dataset->id());
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(IndexingRequest::forDataSet($dataset), $item->data);
    $new_label = $this->randomMachineName();
    $dataset->name = $new_label;
    $dataset->save();
    // Should still be one item, as the old one is now invalid and has been
    // deleted.
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(IndexingRequest::forDataSet($dataset), $item->data);

    // Assert saving while syncing does not queue.
    $queue->deleteQueue();
    $this->assertEquals(0, $queue->numberOfItems());
    $dataset->setSyncing(TRUE);
    $dataset->save();
    $this->assertEquals(0, $queue->numberOfItems());
    $dataset->setSyncing(FALSE);
    $dataset->save();
    $this->assertEquals(1, $queue->numberOfItems());

    $dataset->delete();
    // Should still be one item, as the old one is now invalid and has been
    // deleted.
    $this->assertEquals(1, $queue->numberOfItems());
    $item = $queue->claimItem();
    $this->assertEquals(IndexingRequest::deleteDataSet($dataset), $item->data);
  }

  /**
   * Tests indexing.
   */
  public function testIndexerQueueWorker() {
    $dataset = $this->createTestDataset();
    $queue_id = 'data_pipelines_index:' . $dataset->id();
    $queue_manager = \Drupal::service('plugin.manager.queue_worker');
    $queue = \Drupal::queue($queue_id);
    $this->assertEquals(1, $queue->numberOfItems());
    assert($queue_manager instanceof QueueWorkerManagerInterface);
    $this->assertTrue($queue_manager->hasDefinition($queue_id));
    $instance = $queue_manager->createInstance($queue_id);
    assert($instance instanceof Indexer);
    $instance->processItem($queue->claimItem()->data);
    $indexer = \Drupal::service('data_pipelines.indexer');
    assert($indexer instanceof MemoryDatasetIndexer);
    $this->assertEquals([$dataset->uuid() => iterator_to_array($dataset->data())], $indexer->getIndexedData());
    $dataset->delete();
    $this->assertEquals(1, $queue->numberOfItems());
    $instance->processItem($queue->claimItem()->data);
    $this->assertEquals([], $indexer->getIndexedData());
  }

}
