<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel\Transform;

use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\data_pipelines\DatasetData;
use Drupal\data_pipelines\Entity\Dataset;
use Drupal\Tests\data_pipelines\Kernel\DatasetKernelTestBase;
use Psr\Log\LoggerInterface;

/**
 * Defines a class for testing transform functionality.
 *
 * @group data_pipelines
 *
 * @covers \Drupal\data_pipelines\DatasetPipelineBase
 * @covers \Drupal\data_pipelines\Plugin\DatasetTransform\Concat
 * @covers \Drupal\data_pipelines\Plugin\DatasetTransform\MapValues
 * @covers \Drupal\data_pipelines\Entity\Dataset
 * @covers \Drupal\data_pipelines\Transform\TransformPluginBase
 * @covers \Drupal\data_pipelines\Transform\DatasetTransformPluginManager
 * @covers \Drupal\data_pipelines\DatasetPipelinePluginManager
 */
class TransformTest extends DatasetKernelTestBase implements LoggerInterface {

  use RfcLoggerTrait;

  /**
   * Log.
   *
   * @var array
   */
  protected $log = [];

  /**
   * Tests transform.
   */
  public function testTransform() {
    $file = $this->getTestFile(dirname(__DIR__, 2) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'test_pipeline_1',
      'csv_file' => $file,
    ]);
    $data = iterator_to_array($dataset->data());
    $this->assertCount(2, $data);
    $this->assertEquals([
      new DatasetData([
        'should_we' => TRUE,
        'firstname' => 'joe',
        'lastname' => 'bloggs',
        'full_name' => 'bloggs, joe',
      ]),
      new DatasetData([
        'should_we' => FALSE,
        'firstname' => 'betty',
        'lastname' => 'bloggs',
        'full_name' => 'bloggs, betty',
      ]),
    ], $data);
  }

  /**
   * Tests pipeline with invalid transform.
   */
  public function testInvalidTransform() {
    $this->container->get('logger.factory')->addLogger($this);
    $file = $this->getTestFile(dirname(__DIR__, 2) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'invalid_pipeline_1',
      'csv_file' => $file,
    ]);
    iterator_to_array($dataset->data());
    $this->assertNotEmpty($this->log);
    [$message] = reset($this->log[RfcLogLevel::ERROR]);
    $this->assertEquals('Invalid plugin ID no_such_plugin for transform.', $message);
    [$message] = reset($this->log[RfcLogLevel::WARNING]);
    $this->assertEquals('Missing plugin ID for transform.', $message);
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []) {
    if ($context['channel'] === 'data_pipelines') {
      $this->log[$level][] = [$message, $context];
    }
  }

  /**
   * Tests pipeline with a field plugin for a record and vice versa.
   */
  public function testPluginWithIncorrectRecordAndFieldSupport() {
    $this->container->get('logger.factory')->addLogger($this);
    $file = $this->getTestFile(dirname(__DIR__, 2) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv_file',
      'name' => $this->randomMachineName(),
      'pipeline' => 'invalid_pipeline_2',
      'csv_file' => $file,
    ]);
    $data = iterator_to_array($dataset->data());
    $this->assertCount(2, $data);
    // Data should be unchanged.
    $this->assertEquals([
      new DatasetData([
        'should_we' => 'Y',
        'firstname' => 'joe',
        'lastname' => 'bloggs',
      ]),
      new DatasetData([
        'should_we' => 'N',
        'firstname' => 'betty',
        'lastname' => 'bloggs',
      ]),
    ], $data);
    $this->assertNotEmpty($this->log);
    [$message] = reset($this->log[RfcLogLevel::WARNING]);
    $this->assertEquals('Using transform map to transform records is not support.', $message);
    [$message] = next($this->log[RfcLogLevel::WARNING]);
    $this->assertEquals('Using transform concat to transform fields is not support.', $message);
  }

}
